@@ -1,27 +1,17 @@
-from typing import List, Tuple, Optional
-from datetime import datetime, date, timedelta, timezone
 import logging
-import sys
-import pandas as pd  # type: ignore
-import numpy as np  # type: ignore
 import subprocess
+import sys
 import tempfile
-from pathlib import Path
+from datetime import date, datetime, timedelta, timezone
 from io import StringIO
 from math import floor
+from pathlib import Path
+from typing import List, Optional, Tuple
 
-from worklog.utils import (
-    format_timedelta,
-    empty_df_from_schema,
-    get_datetime_cols_from_schema,
-    check_order_session,
-    sentinel_datetime,
-    get_active_task_ids,
-    get_all_task_ids,
-    extract_intervals,
-    get_pager,
-    calc_log_time,
-)
+import numpy as np  # type: ignore
+import pandas as pd  # type: ignore
+
+from worklog.breaks import AutoBreak
 from worklog.constants import (
     COL_CATEGORY,
     COL_COMMIT_DATETIME,
@@ -31,9 +21,22 @@ from worklog.constants import (
     DEFAULT_LOGGER_NAME,
     LOCAL_TIMEZONE,
     TOKEN_SESSION,
-    TOKEN_TASK,
     TOKEN_START,
     TOKEN_STOP,
+    TOKEN_TASK,
+)
+from worklog.utils import (
+    calc_log_time,
+    check_order_session,
+    empty_df_from_schema,
+    extract_intervals,
+    format_timedelta,
+    get_active_task_ids,
+    get_all_task_ids,
+    get_datetime_cols_from_schema,
+    get_pager,
+    sentinel_datetime,
+    format_numpy_timedelta,
 )
 
 logger = logging.getLogger(DEFAULT_LOGGER_NAME)
@@ -66,6 +69,8 @@ class Log(object):
         "Stop running tasks first: {active_tasks:} or use --force flag.\n"
     )
 
+    auto_break: AutoBreak = AutoBreak()
+
     def __init__(self, fp: str, separator: str = "|") -> None:
         self._log_fp = fp
         self._separator = separator
@@ -73,6 +78,231 @@ class Log(object):
         Path(self._log_fp).touch(mode=0o660)
         self._read()
 
+    def commit(
+        self,
+        category: str,
+        type_: str,
+        offset_min: int = 0,
+        time: Optional[str] = None,
+        identifier: str = None,
+        force: bool = False,
+    ) -> None:
+        """Commit a session/task change to the logfile."""
+        log_date = calc_log_time(offset_min, time)
+        self._commit(category, type_, log_date, identifier, force)
+
+    def doctor(self) -> None:
+        """Test if the logfile is consistent."""
+        self._log_df.groupby(["date"]).apply(
+            lambda group: check_order_session(group, logger)
+        )
+
+    def list_tasks(self):
+        """List all known tasks, i.e. tasks that have been used previously
+        and are stored in the logfile."""
+        task_df = self._log_df[self._log_df[COL_CATEGORY] == TOKEN_TASK]
+        sys.stdout.write("These tasks are listed in the log:\n")
+        for task_id in sorted(task_df[COL_TASK_IDENTIFIER].unique()):
+            sys.stdout.write(f"{task_id}\n")
+
+    def log(self, n: int, use_pager: bool, filter_category: List[str]) -> None:
+        """Display the content of the logfile."""
+        if self._log_df.shape[0] == 0:
+            sys.stdout.write("No data available\n")
+            return
+
+        fields = ["date", "time", COL_CATEGORY, COL_TYPE, COL_TASK_IDENTIFIER]
+        df = self._log_df[fields].iloc[::-1]  # sort in reverse (latest first)
+        df[COL_TASK_IDENTIFIER] = df[COL_TASK_IDENTIFIER].fillna("-")
+        if filter_category:
+            df = df[df[COL_CATEGORY] == filter_category]
+        if n > 0:
+            df = df.head(n=n)
+        if not use_pager:
+            sys.stdout.write(df.to_string(index=False) + "\n")
+        else:
+            with tempfile.NamedTemporaryFile(mode="w") as fh:
+                logger.debug(f"Write content to temporary file: {fh.name}")
+                fh.write(df.to_string(index=False))
+                fh.flush()
+                pager = get_pager()
+                if pager is None:
+                    sys.stdout.write(df.to_string(index=False) + "\n")
+                else:
+                    logger.debug(f"Set pager to {pager}")
+                    process = subprocess.Popen([pager, fh.name])
+                    process.wait()
+
+    def report(self, month_from: datetime, month_to: datetime):
+        """Generate a daily, weekly, monthly and task based report based on
+        the content in the logfile."""
+        session_mask = self._log_df[COL_CATEGORY] == TOKEN_SESSION
+        task_mask = self._log_df[COL_CATEGORY] == TOKEN_TASK
+        time_mask = (self._log_df[COL_LOG_DATETIME] >= month_from) & (
+            self._log_df[COL_LOG_DATETIME] < month_to
+        )
+
+        # Day aggregation
+        df_day = self._aggregate_time(time_mask & session_mask, resample="D")
+        df_day["break"] = df_day["agg_time"].map(self.auto_break.get_duration)
+
+        # Week aggregation
+        df_week = df_day.set_index(COL_LOG_DATETIME).resample("W").sum().reset_index()
+
+        # Month aggregration
+        df_month = df_day.set_index(COL_LOG_DATETIME).resample("M").sum().reset_index()
+
+        for df in (df_day, df_week, df_month):
+            df["agg_time_bookable"] = df["agg_time"] - df["break"]
+
+        # Task aggregation
+        df_tasks = self._aggregate_tasks(time_mask & task_mask)
+
+        print_cols = [COL_LOG_DATETIME, "agg_time"]
+        print_cols_labels = ["Date", "Total time"]
+        if self.auto_break.active:
+            print_cols += ["break", "agg_time_bookable"]
+            print_cols_labels += ["Break", "Bookable time"]
+
+        def _formatters(date: str = "M"):
+            date_max_len = len("2000-01") if date == "M" else len("2000-01-01")
+            return {
+                COL_LOG_DATETIME: lambda v: str(v.date())[:date_max_len],
+                "agg_time": format_numpy_timedelta,
+                "agg_time_bookable": format_numpy_timedelta,
+            }
+
+        self._print_aggregation(
+            "month",
+            df_month,
+            print_cols,
+            print_cols_labels,
+            formatters=_formatters("M"),
+        )
+        self._print_aggregation(
+            "week", df_week, print_cols, print_cols_labels, formatters=_formatters("D"),
+        )
+        self._print_aggregation(
+            "day", df_day, print_cols, print_cols_labels, formatters=_formatters("D")
+        )
+
+        print_cols = [COL_TASK_IDENTIFIER, "agg_time"]
+        print_cols_labels = ["Task name", "Total time"]
+        self._print_aggregation("tasks", df_tasks, print_cols, print_cols_labels)
+
+    def status(
+        self, hours_target: float, hours_max: float, query_date: date, fmt: str = None,
+    ) -> None:
+        """Display the current working status, e.g. total time worked at this
+        day, remaining time, etc."""
+        self._check_nonempty_or_exit(fmt)
+
+        df_day = self._filter_date_category_limit_cols(query_date)
+
+        if df_day.shape[0] == 0:
+            if fmt is None:
+                msg = self._err_msg_log_data_missing_for_date.format(
+                    query_date=query_date
+                )
+                sys.stderr.write(msg)
+            else:
+                sys.stdout.write(self._err_msg_log_data_missing_for_date_short)
+            return
+
+        is_active = self._is_active(df_day)
+        logger.debug(f"Is active: {is_active}")
+
+        df_day = self._add_sentinel(query_date, df_day)
+        facts = self._calc_facts(df_day, hours_target, hours_max)
+
+        all_touched_tasks = get_all_task_ids(self._log_df, query_date)
+        active_tasks = get_active_task_ids(self._log_df, query_date)
+
+        lines = [
+            ("Status", "Tracking {status}"),
+            ("Total time", "{total_time} ({percentage:3}%)"),
+            ("Remaining time", "{remaining_time} ({percentage_remaining:3}%)"),
+            ("Overtime", "{overtime} ({percentage_overtime:3}%)"),
+            ("Break Duration", "{break_duration}"),
+            ("All touched tasks", "{all_touched_tasks}",),
+            ("Active tasks", "{active_tasks}",),
+        ]
+
+        if is_active and date == "today":
+            lines += [("End of work", facts["end_time_str"],)]
+
+        key_max_len = max([len(line[0]) for line in lines])
+        fmt_string = "{:" + str(key_max_len + 1) + "s}: {}"
+
+        stdout_fmt = "\n".join(fmt_string.format(*line) for line in lines) + "\n"
+
+        sys.stdout.write(
+            (stdout_fmt if fmt is None else fmt).format(
+                **facts,
+                status="on" if is_active else "off",
+                active_tasks=f"({len(active_tasks)}) [" + ", ".join(active_tasks) + "]",
+                all_touched_tasks=f"({len(all_touched_tasks)}) ["
+                + ", ".join(all_touched_tasks)
+                + "]",
+            )
+        )
+
+    def stop_active_tasks(self, log_dt: datetime):
+        """Stop all active tasks by commiting changes to the logfile."""
+        query_date = log_dt.date()
+        active_task_ids = get_active_task_ids(self._log_df, query_date)
+        for task_id in active_task_ids:
+            self._commit(TOKEN_TASK, TOKEN_STOP, log_dt, identifier=task_id)
+
+    def task_report(self, task_id):
+        """Generate a report of a given task."""
+        task_mask = self._log_df[COL_CATEGORY] == TOKEN_TASK
+        task_id_mask = self._log_df[COL_TASK_IDENTIFIER] == task_id
+        mask = task_mask & task_id_mask
+        task_df = self._log_df[mask]
+
+        if task_df.shape[0] == 0:
+            sys.stderr.write(
+                (
+                    f"Task ID {task_id} is unknown. "
+                    "See 'wl task list' to list all known tasks.\n"
+                )
+            )
+            exit(1)
+
+        intervals = extract_intervals(task_df, logger=logger)
+
+        intervals_detailed = intervals[["date", "start", "stop", "interval"]].rename(
+            columns={
+                "date": "Date",
+                "start": "Start",
+                "stop": "Stop",
+                "interval": "Duration",
+            }
+        )
+        print("Log entries:\n")
+        print(
+            intervals_detailed.to_string(
+                index=False,
+                formatters={
+                    "Start": lambda x: x.strftime("%H:%M:%S"),
+                    "Stop": lambda x: x.strftime("%H:%M:%S"),
+                    "Duration": lambda x: format_timedelta(
+                        timedelta(microseconds=int(x) / 1e3)
+                    ),
+                },
+            )
+        )
+
+        print("---")
+        print("Daily aggregated:\n")
+        intervals_daily = intervals.groupby(by="date")[["interval"]].sum()
+        intervals_daily.index.name = "Date"
+        intervals_daily = intervals_daily.rename(columns={"interval": "Duration"})
+        print(intervals_daily.to_string())
+
+        print(f"---\nTotal: {intervals_detailed['Duration'].sum()}")
+
     def _read(self) -> None:
         """
         Read data from input file.
@@ -162,23 +392,6 @@ class Log(object):
         # Update sorting of values in-memory.
         self._log_df = self._log_df.sort_values(by=[COL_LOG_DATETIME])
 
-    def commit(
-        self,
-        category: str,
-        type_: str,
-        offset_min: int = 0,
-        time: Optional[str] = None,
-        identifier: str = None,
-        force: bool = False,
-    ) -> None:
-        log_date = calc_log_time(offset_min, time)
-        self._commit(category, type_, log_date, identifier, force)
-
-    def doctor(self) -> None:
-        self._log_df.groupby(["date"]).apply(
-            lambda group: check_order_session(group, logger)
-        )
-
     def _is_active(self, df: pd.DataFrame):
         """
         Returns True if the last entry in a given pandas DataFrame has 
@@ -245,8 +458,10 @@ class Log(object):
         total_time = (df[stop_mask][COL_LOG_DATETIME] - shifted_dt[stop_mask]).sum()
         total_time_str = format_timedelta(total_time)
 
-        hours_target_dt = timedelta(hours=hours_target)
-        hours_max_dt = timedelta(hours=hours_max)
+        break_duration = self.auto_break.get_duration(total_time)
+
+        hours_target_dt = timedelta(hours=hours_target) + break_duration
+        hours_max_dt = timedelta(hours=hours_max) + break_duration
 
         now = datetime.now(timezone.utc).astimezone().replace(microsecond=0)
         end_time = now + (hours_target_dt - total_time)
@@ -278,143 +493,9 @@ class Log(object):
             overtime=overtime_str,
             overtime_short=overtime_str[: len("00:00")],
             percentage_overtime=percentage_overtime,
+            break_duration=format_timedelta(break_duration),
         )
 
-    def status(
-        self, hours_target: float, hours_max: float, query_date: date, fmt: str = None
-    ) -> None:
-        self._check_nonempty_or_exit(fmt)
-
-        df_day = self._filter_date_category_limit_cols(query_date)
-
-        if df_day.shape[0] == 0:
-            if fmt is None:
-                msg = self._err_msg_log_data_missing_for_date.format(
-                    query_date=query_date
-                )
-                sys.stderr.write(msg)
-            else:
-                sys.stdout.write(self._err_msg_log_data_missing_for_date_short)
-            return
-
-        is_active = self._is_active(df_day)
-        logger.debug(f"Is active: {is_active}")
-
-        df_day = self._add_sentinel(query_date, df_day)
-        facts = self._calc_facts(df_day, hours_target, hours_max)
-
-        all_touched_tasks = get_all_task_ids(self._log_df, query_date)
-        active_tasks = get_active_task_ids(self._log_df, query_date)
-
-        lines = [
-            ("Status", "Tracking {status}"),
-            ("Total time", "{total_time} ({percentage:3}%)"),
-            ("Remaining time", "{remaining_time} ({percentage_remaining:3}%)"),
-            ("Overtime", "{overtime} ({percentage_overtime:3}%)"),
-            ("All touched tasks", "{all_touched_tasks}",),
-            ("Active tasks", "{active_tasks}",),
-        ]
-
-        if is_active and date == "today":
-            lines += [("End of work", facts["end_time_str"],)]
-
-        key_max_len = max([len(line[0]) for line in lines])
-        fmt_string = "{:" + str(key_max_len + 1) + "s}: {}"
-
-        stdout_fmt = "\n".join(fmt_string.format(*line) for line in lines) + "\n"
-
-        sys.stdout.write(
-            (stdout_fmt if fmt is None else fmt).format(
-                **facts,
-                status="on" if is_active else "off",
-                active_tasks=f"({len(active_tasks)}) [" + ", ".join(active_tasks) + "]",
-                all_touched_tasks=f"({len(all_touched_tasks)}) ["
-                + ", ".join(all_touched_tasks)
-                + "]",
-            )
-        )
-
-    def log(self, n: int, use_pager: bool, filter_category: List[str]) -> None:
-        if self._log_df.shape[0] == 0:
-            sys.stdout.write("No data available\n")
-            return
-
-        fields = ["date", "time", COL_CATEGORY, COL_TYPE, COL_TASK_IDENTIFIER]
-        df = self._log_df[fields].iloc[::-1]  # sort in reverse (latest first)
-        df[COL_TASK_IDENTIFIER] = df[COL_TASK_IDENTIFIER].fillna("-")
-        if filter_category:
-            df = df[df[COL_CATEGORY] == filter_category]
-        if n > 0:
-            df = df.head(n=n)
-        if not use_pager:
-            sys.stdout.write(df.to_string(index=False) + "\n")
-        else:
-            with tempfile.NamedTemporaryFile(mode="w") as fh:
-                logger.debug(f"Write content to temporary file: {fh.name}")
-                fh.write(df.to_string(index=False))
-                fh.flush()
-                pager = get_pager()
-                if pager is None:
-                    sys.stdout.write(df.to_string(index=False) + "\n")
-                else:
-                    logger.debug(f"Set pager to {pager}")
-                    process = subprocess.Popen([pager, fh.name])
-                    process.wait()
-
-    def list_tasks(self):
-        task_df = self._log_df[self._log_df[COL_CATEGORY] == TOKEN_TASK]
-        sys.stdout.write("These tasks are listed in the log:\n")
-        for task_id in sorted(task_df[COL_TASK_IDENTIFIER].unique()):
-            sys.stdout.write(f"{task_id}\n")
-
-    def task_report(self, task_id):
-        task_mask = self._log_df[COL_CATEGORY] == TOKEN_TASK
-        task_id_mask = self._log_df[COL_TASK_IDENTIFIER] == task_id
-        mask = task_mask & task_id_mask
-        task_df = self._log_df[mask]
-
-        if task_df.shape[0] == 0:
-            sys.stderr.write(
-                (
-                    f"Task ID {task_id} is unknown. "
-                    "See 'wl task list' to list all known tasks.\n"
-                )
-            )
-            exit(1)
-
-        intervals = extract_intervals(task_df, logger=logger)
-
-        intervals_detailed = intervals[["date", "start", "stop", "interval"]].rename(
-            columns={
-                "date": "Date",
-                "start": "Start",
-                "stop": "Stop",
-                "interval": "Duration",
-            }
-        )
-        print("Log entries:\n")
-        print(
-            intervals_detailed.to_string(
-                index=False,
-                formatters={
-                    "Start": lambda x: x.strftime("%H:%M:%S"),
-                    "Stop": lambda x: x.strftime("%H:%M:%S"),
-                    "Duration": lambda x: format_timedelta(
-                        timedelta(microseconds=int(x) / 1e3)
-                    ),
-                },
-            )
-        )
-
-        print("---")
-        print("Daily aggregated:\n")
-        intervals_daily = intervals.groupby(by="date")[["interval"]].sum()
-        intervals_daily.index.name = "Date"
-        intervals_daily = intervals_daily.rename(columns={"interval": "Duration"})
-        print(intervals_daily.to_string())
-
-        print(f"---\nTotal: {intervals_detailed['Duration'].sum()}")
-
     def _aggregate_base(self, mask, keep_cols: List[str] = []):
         df = self._log_df[mask]
         shifted_dt = df[COL_LOG_DATETIME].shift(1)
@@ -433,10 +514,6 @@ class Log(object):
             .reset_index()
             .dropna()
         )
-        len_date = len("2000-01-01" if resample == "D" else "2000-01")
-        df_day["date"] = df_day[COL_LOG_DATETIME].apply(
-            lambda x: str(x.date())[:len_date]
-        )
         return df_day
 
     def _aggregate_tasks(self, mask):
@@ -448,63 +525,18 @@ class Log(object):
             .reset_index()
         )
 
-    def stop_active_tasks(self, log_dt: datetime):
-        query_date = log_dt.date()
-        active_task_ids = get_active_task_ids(self._log_df, query_date)
-        for task_id in active_task_ids:
-            self._commit(TOKEN_TASK, TOKEN_STOP, log_dt, identifier=task_id)
-
-    def report(self, month_from: datetime, month_to: datetime):
-        session_mask = self._log_df[COL_CATEGORY] == TOKEN_SESSION
-        task_mask = self._log_df[COL_CATEGORY] == TOKEN_TASK
-        time_mask = (self._log_df[COL_LOG_DATETIME] >= month_from) & (
-            self._log_df[COL_LOG_DATETIME] < month_to
-        )
-
-        def _time_repr(value: timedelta) -> str:
-            hours = floor(value.total_seconds() / 3600)
-            minutes = floor((value.total_seconds() - hours * 3600) / 60)
-            seconds = floor(value.total_seconds() % 60)
-            return "{hours:02}:{minutes:02}:{seconds:02}".format(
-                hours=hours, minutes=minutes, seconds=seconds
-            )
-
-        df_month = self._aggregate_time(time_mask & session_mask, resample="M")
-        df_month["agg_time_custom"] = df_month["agg_time"].map(_time_repr)
-
-        # Day aggregation
-        df_day = self._aggregate_time(time_mask & session_mask, resample="D")
-        df_day["agg_time_custom"] = df_day["agg_time"].map(_time_repr)
-
-        # Task aggregation
-        df_tasks = self._aggregate_tasks(time_mask & task_mask)
-        df_tasks["agg_time_custom"] = df_tasks["agg_time"].map(_time_repr)
-
-        print("Aggregated by month:")
-        print("--------------------")
+    def _print_aggregation(self, agg_label, df, cols, col_titles, formatters=None):
+        headline = f"Aggregated by {agg_label}:"
+        print(headline)
+        print("-" * len(headline))
         print(
-            df_month[["date", "agg_time_custom"]].to_string(
-                index=False, header=["Date", "Total time"]
+            df.to_string(
+                index=False,
+                columns=cols,
+                header=col_titles,
+                formatters=formatters,
+                col_space=20,
             )
         )
 
         print()
-
-        print("Aggregated by day:")
-        print("------------------")
-        print(
-            df_day[["date", "agg_time_custom"]].to_string(
-                index=False, header=["Date", "Total time"]
-            )
-        )
-
-        print()
-
-        print("Aggregated by tasks:")
-        print("--------------------")
-        print(
-            df_tasks[[COL_TASK_IDENTIFIER, "agg_time_custom"]].to_string(
-                index=False, header=["Task name", "Total time"]
-            )
-        )
-
