@@ -1,4 +1,4 @@
-from typing import List, Iterable, Tuple, Dict, Optional
+from typing import List, Iterable, Tuple, Dict, Optional, Union
 from pandas import DataFrame, Series  # type: ignore
 import logging
 import sys
@@ -7,6 +7,8 @@ import os
 from functools import reduce
 from datetime import datetime, date, timezone, timedelta, tzinfo
 import shutil
+from math import floor
+import numpy as np
 
 from worklog.constants import (
     COL_CATEGORY,
@@ -179,3 +181,16 @@ def calc_log_time(offset_min: int = 0, time: Optional[str] = None) -> datetime:
         my_date = _get_or_update_dt(my_date, time)
 
     return my_date
+
+
+def format_numpy_timedelta(value: np.timedelta64) -> str:
+    seconds = value / np.timedelta64(1, "s")
+    if np.isnan(seconds):
+        hours = minutes = seconds = 0
+    else:
+        hours = floor(seconds / 3600)
+        minutes = floor((seconds - hours * 3600) / 60)
+        seconds = floor(seconds % 60)
+    return "{hours:02}:{minutes:02}:{seconds:02}".format(
+        hours=hours, minutes=minutes, seconds=seconds
+    )
